@@ -16,8 +16,7 @@ use prost::{
 
 use self::{field::DynamicMessageFieldSet, unknown::UnknownFieldSet};
 use crate::{
-    descriptor::Kind, ExtensionDescriptor, FieldDescriptor, MessageDescriptor, OneofDescriptor,
-    ReflectMessage,
+    descriptor::Kind, ExtensionDescriptor, FieldDescriptor, MessageDescriptor, ReflectMessage,
 };
 
 /// [`DynamicMessage`] provides encoding, decoding and reflection of a protobuf message.
@@ -112,9 +111,9 @@ impl DynamicMessage {
 
     /// Returns `true` if this message has the given field set.
     ///
-    /// If the field type supports distinguishing whether a value has been set, such as
-    /// for messages, then this method returns `true` only if a value has been set. For other
-    /// types, such as integers, it returns `true` if the value is set to a non-default value.
+    /// If the field type supports distinguishing whether a value has been set (see [`supports_presence`][FieldDescriptor::supports_presence]),
+    /// such as for messages, then this method returns `true` only if a value has been set. For
+    /// other types, such as integers, it returns `true` if the value is set to a non-default value.
     ///
     /// If this method returns `false`, then the field will not be included in the encoded bytes
     /// of this message.
@@ -135,17 +134,6 @@ impl DynamicMessage {
     /// by [`Value::is_valid_for_field`].
     pub fn set_field(&mut self, field_desc: &FieldDescriptor, value: Value) {
         self.fields.set(field_desc, value);
-        if let Some(oneof_desc) = field_desc.containing_oneof() {
-            self.clear_oneof_fields(oneof_desc, field_desc.number());
-        }
-    }
-
-    fn clear_oneof_fields(&mut self, oneof_desc: OneofDescriptor, set_field: u32) {
-        for oneof_field in oneof_desc.fields() {
-            if oneof_field.number() != set_field {
-                self.clear_field(&oneof_field);
-            }
-        }
     }
 
     /// Clears the given field.
@@ -218,7 +206,7 @@ impl DynamicMessage {
             .map(|field_desc| self.get_field(&field_desc))
     }
 
-    /// Sets the value of the field with name `name`, or the default value if it is unset.
+    /// Sets the value of the field with name `name`.
     ///
     /// If no field with the given name exists, this method does nothing.
     ///
@@ -249,21 +237,19 @@ impl DynamicMessage {
             .unwrap_or(false)
     }
 
-    /// Gets the value of the give extension field, or the default value if it is unset.
+    /// Gets the value of the given extension field, or the default value if it is unset.
     ///
     /// See [`get_field`][Self::get_field] for more details.
-    pub fn get_extension(&self, extension_desc: &ExtensionDescriptor) -> Option<Cow<'_, Value>> {
+    pub fn get_extension(&self, extension_desc: &ExtensionDescriptor) -> Cow<'_, Value> {
         match self.extension_fields() {
-            Some(extensions) => Some(extensions.get(extension_desc)),
-            None => Some(Cow::Owned(Value::default_value_for_extension(
-                extension_desc,
-            ))),
+            Some(extensions) => extensions.get(extension_desc),
+            None => Cow::Owned(Value::default_value_for_extension(extension_desc)),
         }
     }
 
-    /// Sets the value of the given extension field, or the default value if it is unset.
+    /// Sets the value of the given extension field.
     ///
-    /// See [`clear_field`][Self::clear_field] for more details.
+    /// See [`set_field`][Self::set_field] for more details.
     pub fn set_extension(&mut self, extension_desc: &ExtensionDescriptor, value: Value) {
         self.extension_fields_mut().set(extension_desc, value)
     }
@@ -340,6 +326,13 @@ impl ReflectMessage for DynamicMessage {
     fn descriptor(&self) -> MessageDescriptor {
         self.desc.clone()
     }
+
+    fn transcode_to_dynamic(&self) -> DynamicMessage
+    where
+        Self: Sized,
+    {
+        self.clone()
+    }
 }
 
 impl Value {
